# -*- coding: utf-8 -*-
"""
    country

    Country

    :copyright: (c) 2013 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
from trytond.pool import PoolMeta


__all__ = ['Subdivision']
__metaclass__ = PoolMeta


class Subdivision:
    "Subdivision"
    __name__ = 'country.subdivision'

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(Subdivision, cls).__setup__()
        cls._error_messages.update({
            'state_not_found': 'State %s does not exist in country %s.',
        })

    @classmethod
    def search_using_ebay_state(cls, value, country):
        """
        Searches for state with given ebay StateOrProvince value.

        :param value: Code or Name of state from ebay
        :param country: Active record of country
        :return: Active record of state if found else raises error
        """
        subdivisions = cls.search([
            ('country', '=', country.id),
            ('code', '=', country.code + '-' + value)
        ])

        if not subdivisions:
            subdivisions = cls.search([
                ('country', '=', country.id),
                ('name', 'ilike', value),
            ], limit=1)

        if not subdivisions:
            return cls.raise_user_error(
                "state_not_found", error_args=(value, country.name)
            )

        return subdivisions[0]



# -*- coding: UTF-8 -*-
'''
    product

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
'''
from trytond.transaction import Transaction
from trytond.pool import PoolMeta, Pool
from decimal import Decimal


__all__ = [
    'Product',
]
__metaclass__ = PoolMeta


class Product:
    "Product"

    __name__ = "product.product"

    @classmethod
    def extract_product_values_from_ebay_data(cls, product_data):
        """
        Extract product values from the ebay data, used for
        creation of product. This method can be overwritten by
        custom modules to store extra info to a product

        :param: product_data
        :returns: Dictionary of values
        """
        SaleChannel = Pool().get('sale.channel')

        ebay_channel = SaleChannel(Transaction().context['current_channel'])
        ebay_channel.validate_ebay_channel()
        return {
            'name': product_data['Item']['Title'],
            'default_uom': ebay_channel.default_uom.id,
            'salable': True,
            'sale_uom': ebay_channel.default_uom.id,
        }

    @classmethod
    def create_from(cls, channel, product_data):
        """
        Create product for channel
        """
        if channel.source != 'ebay':
            return super(Product, cls).create_from(channel, product_data)
        return cls.create_using_ebay_data(product_data)

    @classmethod
    def create_using_ebay_data(cls, product_data):
        """
        Create a new product with the `product_data` from ebay.

        :param product_data: Product Data from eBay
        :returns: Browse record of product created
        """
        Template = Pool().get('product.template')

        product_values = cls.extract_product_values_from_ebay_data(
            product_data
        )

        product_values.update({
            'products': [('create', [{
                'description': product_data['Item']['Description'],
                'list_price': Decimal(
                    product_data['Item']['BuyItNowPrice']['value'] or
                    product_data['Item']['StartPrice']['value']
                ),
                'cost_price':
                    Decimal(product_data['Item']['StartPrice']['value']),
                'code':
                    product_data['Item'].get('SKU', None) and
                    product_data['Item']['SKU'] or None,
            }])],
        })

        product_template, = Template.create([product_values])

        return product_template.products[0]



# -*- coding: utf-8 -*-
"""
    __init__

    :copyright: © 2013 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import unittest
import trytond.tests.test_tryton

from tests.test_views import TestViewDepend
from tests.test_country import TestCountry
from tests.test_party import TestParty
from tests.test_product import TestProduct
from tests.test_sale import TestSale
from tests.test_channel import TestChannel


def suite():
    """
    Define suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests([
        unittest.TestLoader().loadTestsFromTestCase(TestViewDepend),
        unittest.TestLoader().loadTestsFromTestCase(TestCountry),
        unittest.TestLoader().loadTestsFromTestCase(TestParty),
        unittest.TestLoader().loadTestsFromTestCase(TestProduct),
        unittest.TestLoader().loadTestsFromTestCase(TestSale),
        unittest.TestLoader().loadTestsFromTestCase(TestChannel),
    ])
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    test_channel

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import unittest

import trytond.tests.test_tryton
from trytond.tests.test_tryton import USER, DB_NAME, CONTEXT
from test_base import TestBase
from trytond.transaction import Transaction
from trytond.exceptions import UserError


class TestChannel(TestBase):
    """
    Tests Channel
    """

    def test_0010_check_unique_ebay_credentials(self):
        """
        Tests if error is raised for using same ebay credentials more than one
        time
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            warehouse, = self.Location.search([
                ('type', '=', 'warehouse')
            ], limit=1)

            # Manual channel with none values
            self.SaleChannel.create([{
                'name': 'eBay Account',
                'warehouse': warehouse.id,
                'company': self.company.id,
                'source': 'manual',
                'currency': self.company.currency.id,
                'price_list': self.price_list,
                'invoice_method': 'manual',
                'shipment_method': 'manual',
                'payment_term': self.payment_term,
                'ebay_app_id': None,
                'ebay_dev_id': None,
                'ebay_cert_id': None,
                'ebay_token': None,
                'is_ebay_sandbox': False,
                'default_uom': self.uom.id,
            }])

            # Manual channel again with none values should not raise error
            self.SaleChannel.create([{
                'name': 'eBay Account',
                'warehouse': warehouse.id,
                'company': self.company.id,
                'source': 'manual',
                'currency': self.company.currency.id,
                'price_list': self.price_list,
                'invoice_method': 'manual',
                'shipment_method': 'manual',
                'payment_term': self.payment_term,
                'ebay_app_id': None,
                'ebay_dev_id': None,
                'ebay_cert_id': None,
                'ebay_token': None,
                'is_ebay_sandbox': False,
                'default_uom': self.uom.id,
            }])

            # eBay channel with credentials
            self.SaleChannel.create([{
                'name': 'eBay Account',
                'warehouse': warehouse.id,
                'company': self.company.id,
                'source': 'ebay',
                'currency': self.company.currency.id,
                'price_list': self.price_list,
                'invoice_method': 'manual',
                'shipment_method': 'manual',
                'payment_term': self.payment_term,
                'ebay_app_id': 'test_app_id',
                'ebay_dev_id': 'test_dev_id',
                'ebay_cert_id': 'test_cert_id',
                'ebay_token': 'a long test token',
                'is_ebay_sandbox': True,
                'default_uom': self.uom.id,
            }])

            # eBay channel with same credentials again should raise error
            with self.assertRaises(UserError):
                self.SaleChannel.create([{
                    'name': 'eBay Account',
                    'warehouse': warehouse.id,
                    'company': self.company.id,
                    'source': 'ebay',
                    'currency': self.company.currency.id,
                    'price_list': self.price_list,
                    'invoice_method': 'manual',
                    'shipment_method': 'manual',
                    'payment_term': self.payment_term,
                    'ebay_app_id': 'test_app_id',
                    'ebay_dev_id': 'test_dev_id',
                    'ebay_cert_id': 'test_cert_id',
                    'ebay_token': 'a long test token',
                    'is_ebay_sandbox': True,
                    'default_uom': self.uom.id,
                }])


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestChannel)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    channel.py

    :copyright: (c) 2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import dateutil.parser
from datetime import datetime

from ebaysdk.trading import Connection as trading
from trytond.transaction import Transaction
from trytond.wizard import Wizard, StateView, Button
from trytond.model import ModelView, fields
from trytond.pool import Pool, PoolMeta
from trytond.pyson import Eval


__all__ = [
    'SaleChannel', 'CheckEbayTokenStatusView', 'CheckEbayTokenStatus',
]
__metaclass__ = PoolMeta

EBAY_STATES = {
    'required': Eval('source') == 'ebay',
    'invisible': ~(Eval('source') == 'ebay')
}


class SaleChannel:
    "Sale Channel"
    __name__ = 'sale.channel'

    ebay_app_id = fields.Char(
        'eBay AppID', states=EBAY_STATES, depends=['source'],
        help="APP ID of the account - provided by eBay",
    )

    ebay_dev_id = fields.Char(
        'eBay DevID', help="Dev ID of account - provided by eBay",
        states=EBAY_STATES, depends=['source']
    )

    ebay_cert_id = fields.Char(
        'eBay CertID', help="Cert ID of account - provided by eBay",
        states=EBAY_STATES, depends=['source']
    )

    ebay_token = fields.Text(
        'eBay Token', states=EBAY_STATES, depends=['source'],
        help="Token for this user account - to be generated from eBay "
        "developer home. If it expirees, then a new one should be generated",
    )

    is_ebay_sandbox = fields.Boolean(
        'Is eBay sandbox ?',
        help="Select this if this account is a sandbox account",
        states={
            'invisible': ~(Eval('source') == 'ebay')
        }, depends=['source']
    )

    @classmethod
    def get_source(cls):
        """
        Get the source
        """
        sources = super(SaleChannel, cls).get_source()

        sources.append(('ebay', 'eBay'))

        return sources

    def get_last_order_import_time_required(self, name):
        """
        Change the getter to make last_order_import_time required for ebay
        """
        if self.source == 'ebay':
            return True

        return super(SaleChannel, self).get_last_order_import_time_required(
            name
        )

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(SaleChannel, cls).__setup__()

        cls._error_messages.update({
            "no_orders":
                'No new orders have been placed on eBay for this '
                'Channel after %s',
            "invalid_channel": "Current channel does not belong to eBay!",
            'same_ebay_credentials':
                'All the ebay credentials should be unique.'
        })
        cls._buttons.update({
            'check_ebay_token_status': {},
        })

    @classmethod
    def validate(cls, channels):
        """
        Validate sale channel
        """
        super(SaleChannel, cls).validate(channels)

        for channel in channels:
            channel.check_unique_app_dev_cert_token()

    def check_unique_app_dev_cert_token(self):
        """
        App ID, Dev ID, Cert ID and Token must be unique
        """
        if not all([
            self.ebay_app_id, self.ebay_dev_id,
            self.ebay_cert_id, self.ebay_token
        ]):
            return
        if self.search([
            ('ebay_app_id', '=', self.ebay_app_id),
            ('ebay_dev_id', '=', self.ebay_dev_id),
            ('ebay_cert_id', '=', self.ebay_cert_id),
            ('ebay_token', '=', self.ebay_token),
            ('id', '!=', self.id)
        ]):
            self.raise_user_error("same_ebay_credentials")

    def get_ebay_trading_api(self):
        """Create an instance of ebay trading api

        :return: ebay trading api instance
        """
        domain = 'api.sandbox.ebay.com' if \
            self.is_ebay_sandbox else 'api.ebay.com'
        return trading(
            appid=self.ebay_app_id,
            certid=self.ebay_cert_id,
            devid=self.ebay_dev_id,
            token=self.ebay_token,
            domain=domain,
            config_file=None,
        )

    @classmethod
    @ModelView.button_action('ebay.wizard_check_ebay_token_status')
    def check_ebay_token_status(cls, channels):
        """
        Check the status of token and display to user
        """
        pass

    def validate_ebay_channel(self):
        """
        Check if current channel belongs to ebay
        """
        if self.source != 'ebay':
            self.raise_user_error("invalid_channel")

    def import_orders(self):
        """
        Downstream implementation of channel.import_orders
        :return: List of active record of sale imported
        """
        if self.source != 'ebay':
            return super(SaleChannel, self).import_orders()

        self.validate_ebay_channel()

        sales = []
        api = self.get_ebay_trading_api()
        now = datetime.utcnow()

        last_import_time = self.last_order_import_time

        # Update current time for order update
        self.write([self], {'last_order_import_time': now})

        response = api.execute(
            'GetOrders', {
                'CreateTimeFrom': last_import_time,
                'CreateTimeTo': now
            }
        ).dict()
        if not response.get('OrderArray'):
            self.raise_user_error(
                'no_orders', (last_import_time, )
            )

        # Orders are returned as dictionary for single order and as
        # list for multiple orders.
        # Convert to list if dictionary is returned
        if isinstance(response['OrderArray']['Order'], dict):
            orders = [response['OrderArray']['Order']]
        else:
            orders = response['OrderArray']['Order']

        with Transaction().set_context({'current_channel': self.id}):
            for order_data in orders:
                sales.append(self.import_order(order_data))

        return sales

    def import_order(self, order_data):
        "Downstream implementation of channel.import_order from sale channel"
        if self.source != 'ebay':
            return super(SaleChannel, self).import_order(order_data)

        Sale = Pool().get('sale.sale')

        sales = Sale.search([
            ('channel_identifier', '=', order_data['OrderID']),
        ])
        if sales:
            return sales[0]

        return Sale.create_using_ebay_data(order_data)

    def import_product(self, sku, product_data=None):
        """
        Import specific product for this ebay channel
        Downstream implementation for channel.import_product
        """
        Product = Pool().get('product.product')
        Listing = Pool().get('product.product.channel_listing')

        if self.source != 'ebay':
            return super(SaleChannel, self).import_product(
                sku, product_data
            )

        # Check if there's a product with SKU (Ebay Custom Label)
        # The SKU on product may be different from that of listing
        # because SKU on a product on ebay is optional and is supplied
        # by user itself.
        existing_listings = Listing.search([
            ('product_identifier', '=', product_data['ItemID']),
            ('channel', '=', self),
        ])
        if existing_listings:
            return existing_listings[0].product

        products = Product.search([('code', '=', sku)])
        self.validate_ebay_channel()
        api = self.get_ebay_trading_api()
        full_product_data = None
        if not products:
            # If product is not found get the info from ebay and
            # delegate to create_from

            full_product_data = full_product_data or \
                api.execute(
                    'GetItem', {
                        'ItemID': product_data['ItemID'],
                        'DetailLevel': 'ReturnAll'
                    }
                ).dict()

            products = [Product.create_from(self, full_product_data)]

        product, = products
        listings = Listing.search([
            ('product', '=', product),
            ('channel', '=', self),
        ])
        if not listings:
            full_product_data = full_product_data or \
                api.execute(
                    'GetItem', {
                        'ItemID': product_data['ItemID'],
                        'DetailLevel': 'ReturnAll'
                    }
                ).dict()
            Listing(
                product=product,
                channel=self,
                product_identifier=full_product_data['Item']['ItemID'],
            ).save()

        return product


class CheckEbayTokenStatusView(ModelView):
    "Check Token Status View"
    __name__ = 'channel.ebay.check_token_status.view'

    status = fields.Char('Status', readonly=True)
    expiry_date = fields.DateTime('Expiry Date', readonly=True)


class CheckEbayTokenStatus(Wizard):
    """
    Check Token Status Wizard

    Check token status for the current ebay channel's token.
    """
    __name__ = 'channel.ebay.check_token_status'

    start = StateView(
        'channel.ebay.check_token_status.view',
        'ebay.check_ebay_token_status_view_form',
        [
            Button('OK', 'end', 'tryton-ok'),
        ]
    )

    def default_start(self, data):
        """
        Check the status of the token of the ebay channel

        :param data: Wizard data
        """
        SaleChannel = Pool().get('sale.channel')

        ebay_channel = SaleChannel(Transaction().context.get('active_id'))

        api = ebay_channel.get_ebay_trading_api()
        response = api.execute('GetTokenStatus').dict()

        return {
            'status': response['TokenStatus']['Status'],
            'expiry_date': dateutil.parser.parse(
                response['TokenStatus']['ExpirationTime']
            ),
        }



# -*- coding: utf-8 -*-
"""
    test_views

    Tests views and depends

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
from trytond.pool import Pool
from .country import Subdivision
from .party import Party, Address
from .product import Product
from .sale import Sale
from channel import (
    SaleChannel, CheckEbayTokenStatusView, CheckEbayTokenStatus,
)


def register():
    "Register classes with pool"
    Pool.register(
        Subdivision,
        Party,
        Address,
        SaleChannel,
        Product,
        Sale,
        CheckEbayTokenStatusView,
        module='ebay', type_='model'
    )
    Pool.register(
        CheckEbayTokenStatus,
        module='ebay', type_='wizard'
    )



# -*- coding: utf-8 -*-
"""
    test_sale

    Tests sale

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import os
import sys
import unittest
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__,
        '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, DB_NAME, CONTEXT
from test_base import TestBase, load_json
from trytond.transaction import Transaction


class TestSale(TestBase):
    """
    Tests import of sale order
    """

    def test_0010_import_sale_order(self):
        """
        Tests import of sale order using ebay data with ebay state as new
        """
        Sale = POOL.get('sale.sale')
        Party = POOL.get('party.party')
        Product = POOL.get('product.product')
        Listing = POOL.get('product.product.channel_listing')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({
                'current_channel': self.ebay_channel.id,
                'company': self.company
            }):

                orders = Sale.search([])
                self.assertEqual(len(orders), 0)

                order_data = load_json(
                    'orders', '283054010'
                )['OrderArray']['Order'][0]

                # We need to have SKU in all items of order_data
                order_data['TransactionArray']['Transaction'][0]['Item'].update(
                    {'SKU': 'test_sku1'}
                )
                order_data['TransactionArray']['Transaction'][1]['Item'].update(
                    {'SKU': 'test_sku2'}
                )

                Party.create_using_ebay_data(
                    load_json('users', 'testuser_ritu123')
                )

                ebay_data1 = load_json('products', '110162956809')
                ebay_data2 = load_json('products', '110162957156')

                # We need to have SKU in ebay_data
                ebay_data1['Item'].update({'SKU': 'test_sku1'})
                ebay_data2['Item'].update({'SKU': 'test_sku2'})

                # Create Product and Listing
                # Listing is required because if product is not found
                # then a new product is created from API request
                product1 = Product.create_using_ebay_data(
                    ebay_data1
                )
                Listing(
                    product=product1,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data1['Item']['ItemID']
                ).save()

                product2 = Product.create_using_ebay_data(
                    ebay_data2
                )
                Listing(
                    product=product2,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data2['Item']['ItemID']
                ).save()

                order1 = self.ebay_channel.import_order(order_data)

                self.assertEqual(order1.state, 'confirmed')

                orders = Sale.search([])
                self.assertEqual(len(orders), 1)

                # Item lines + shipping line should be equal to lines on tryton
                self.assertEqual(len(order1.lines), 3)

                order2 = self.ebay_channel.import_order(order_data)
                orders = Sale.search([])
                self.assertEqual(len(orders), 1)

                self.assertEqual(order1, order2)

    def test_0020_import_sale_order_with_exception(self):
        """
        Tests if exception is created for sale order when order total mismatch
        """
        Sale = POOL.get('sale.sale')
        Party = POOL.get('party.party')
        Product = POOL.get('product.product')
        ChannelException = POOL.get('channel.exception')
        Listing = POOL.get('product.product.channel_listing')

        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            with Transaction().set_context({
                'current_channel': self.ebay_channel.id,
                'company': self.company
            }):

                orders = Sale.search([])
                self.assertEqual(len(orders), 0)

                order_data = load_json(
                    'orders', '283054010'
                )['OrderArray']['Order'][0]

                # We need to have SKU in all items of order_data
                order_data['TransactionArray']['Transaction'][0]['Item'].update(
                    {'SKU': 'test_sku1'}
                )
                order_data['TransactionArray']['Transaction'][1]['Item'].update(
                    {'SKU': 'test_sku2'}
                )

                Party.create_using_ebay_data(
                    load_json('users', 'testuser_ritu123')
                )

                ebay_data1 = load_json('products', '110162956809')
                ebay_data2 = load_json('products', '110162957156')

                # We need to have SKU in ebay_data
                ebay_data1['Item'].update({'SKU': 'test_sku1'})
                ebay_data2['Item'].update({'SKU': 'test_sku2'})

                # Create Product and Listing
                # Listing is required because if product is not found
                # then a new product is created from API request
                product1 = Product.create_using_ebay_data(
                    ebay_data1
                )
                Listing(
                    product=product1,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data1['Item']['ItemID']
                ).save()

                product2 = Product.create_using_ebay_data(
                    ebay_data2
                )
                Listing(
                    product=product2,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data2['Item']['ItemID']
                ).save()

                self.assertEqual(order_data['Total']['value'], '4.0')

                self.assertFalse(ChannelException.search([]))

                # Lets Change order total to make it raise exception
                order_data['Total']['value'] = '8.5'

                order = self.ebay_channel.import_order(order_data)

                self.assertTrue(ChannelException.search([]))

                self.assertTrue(order.has_channel_exception)

                self.assertNotEqual(order.state, 'confirmed')

                orders = Sale.search([])
                self.assertEqual(len(orders), 1)

                # Item lines + shipping line should be equal to lines on tryton
                self.assertEqual(len(order.lines), 3)


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestSale)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    sale

    Sale

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import dateutil.parser
from decimal import Decimal

from trytond.transaction import Transaction
from trytond.pool import PoolMeta, Pool


__all__ = ['Sale']

__metaclass__ = PoolMeta


class Sale:
    "Sale"
    __name__ = 'sale.sale'

    @classmethod
    def find_or_create_using_ebay_id(cls, order_id):
        """
        This method tries to find the sale with the order ID
        first and if not found it will fetch the info from ebay and
        create a new sale with the data from ebay using
        create_using_ebay_data

        :param order_id: Order ID from ebay
        :type order_id: string
        :returns: Active record of sale order created/found
        """
        SaleChannel = Pool().get('sale.channel')

        sales = cls.search([
            ('channel_identifier', '=', order_id),
        ])
        if sales:
            return sales[0]

        ebay_channel = SaleChannel(Transaction().context['current_channel'])
        ebay_channel.validate_ebay_channel()

        api = ebay_channel.get_ebay_trading_api()

        order_data = api.execute(
            'GetOrders', {
                'OrderIDArray': {
                    'OrderID': order_id
                }, 'DetailLevel': 'ReturnAll'
            }
        ).dict()

        return cls.create_using_ebay_data(order_data['OrderArray']['Order'])

    @classmethod
    def create_using_ebay_data(cls, order_data):
        """
        Create a sale from ebay data

        :param order_data: Order data from ebay
                           Ref: http://developer.ebay.com/DevZone/XML/docs/\
                                   Reference/eBay/GetOrders.html#Response
        :return: Active record of record created
        """
        Party = Pool().get('party.party')
        Currency = Pool().get('currency.currency')
        SaleChannel = Pool().get('sale.channel')
        ChannelException = Pool().get('channel.exception')

        ebay_channel = SaleChannel(Transaction().context['current_channel'])

        ebay_channel.validate_ebay_channel()

        currency, = Currency.search([
            ('code', '=', order_data['Total']['_currencyID'])
        ], limit=1)

        # Transaction is similar to order lines
        # In the if..else below we fetch the first item from the array to
        # get the item which will be used to establish a relationship
        # between seller and buyer.
        transaction = order_data['TransactionArray']['Transaction']
        if isinstance(transaction, dict):
            # If its a single line order, then the array will be dict
            item = transaction
        else:
            # In case of multi line orders, the transaction array will be
            # a list of dictionaries
            item = transaction[0]

        # Get an item ID so that ebay can establish a relationship between
        # seller and buyer.
        # eBay has a security feature which allows a seller
        # to fetch the information of a buyer via API only when there is
        # a seller-buyer relationship between both via some item.
        # If this item is not passed, then ebay would not return important
        # informations like eMail etc.
        item_id = item['Item']['ItemID']
        party = Party.find_or_create_using_ebay_id(
            order_data['BuyerUserID'], item_id=item_id
        )

        if order_data['ShippingAddress'].get('Phone', None):
            party.add_phone_using_ebay_data(
                order_data['ShippingAddress']['Phone']
            )

        party_invoice_address = party_shipping_address = \
            party.find_or_create_address_using_ebay_data(
                order_data['ShippingAddress']
            )

        sale_data = {
            'reference': order_data['OrderID'],
            'sale_date': dateutil.parser.parse(
                order_data['CreatedTime'].split()[0]
            ).date(),
            'party': party.id,
            'currency': currency.id,
            'invoice_address': party_invoice_address.id,
            'shipment_address': party_shipping_address.id,
            'channel_identifier': order_data['OrderID'],
            'lines': cls.get_item_line_data_using_ebay_data(order_data),
            'channel': ebay_channel.id,
        }

        sale_data['lines'].append(
            cls.get_shipping_line_data_using_ebay_data(ebay_channel, order_data)
        )

        # TODO: Handle Discounts
        # TODO: Handle Taxes

        sale, = cls.create([sale_data])

        # Create channel exception if order total does not match
        if sale.total_amount != Decimal(order_data['Total']['value']):
            ChannelException.create([{
                'origin': '%s,%s' % (sale.__name__, sale.id),
                'log': 'Order total does not match.',
                'channel': sale.channel.id,
            }])

            return sale

        # We import only completed orders, so we can confirm them all
        cls.quote([sale])
        cls.confirm([sale])

        # TODO: Process the order for invoice as the payment info is received

        return sale

    @classmethod
    def get_item_line_data_using_ebay_data(cls, order_data):
        """
        Make data for an item line from the ebay data.

        :param order_data: Order Data from ebay
        :return: List of data of order lines in required format
        """
        SaleChannel = Pool().get('sale.channel')

        ebay_channel = SaleChannel(Transaction().context['current_channel'])

        ebay_channel.validate_ebay_channel()

        line_data = []
        transaction = order_data['TransactionArray']['Transaction']
        if isinstance(transaction, dict):
            # If its a single line order, then the transaction will be dict
            items = [transaction]
        else:
            # In case of multi line orders, the transaction will be
            # a list of dictionaries
            items = transaction
        for item in items:
            product_data = {
                'ItemID': item['Item']['ItemID'],
            }

            values = {
                'description': item['Item']['Title'],
                'unit_price': Decimal(
                    item['TransactionPrice']['value']
                ),
                'unit': ebay_channel.default_uom.id,
                'quantity': Decimal(
                    item['QuantityPurchased']
                ),
                'channel_identifier': item['OrderLineItemID'],
                'product': ebay_channel.get_product(
                    item['Item']['SKU'],
                    product_data,
                ).id
            }
            line_data.append(('create', [values]))

        return line_data

    @classmethod
    def get_shipping_line_data_using_ebay_data(cls, channel, order_data):
        """
        Create a shipping line for the given sale using ebay data

        :param order_data: Order Data from ebay
        """
        return ('create', [{
            'description': 'eBay Shipping and Handling',
            'unit_price': Decimal(
                order_data['ShippingServiceSelected'].get(
                    'ShippingServiceCost', 0.00
                ) and order_data[
                    'ShippingServiceSelected'
                ]['ShippingServiceCost']['value']
            ),
            'unit': channel.default_uom.id,
            'note': order_data['ShippingServiceSelected'].get(
                'ShippingService', None
            ) and order_data[
                'ShippingServiceSelected']['ShippingService'],
            'quantity': 1,
        }])



# -*- coding: utf-8 -*-
"""
    party

    Party

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
from trytond.model import fields
from trytond.pool import PoolMeta, Pool
from trytond.transaction import Transaction


__all__ = ['Party', 'Address']
__metaclass__ = PoolMeta


class Party:
    "Party"
    __name__ = 'party.party'

    ebay_user_id = fields.Char(
        'eBay User ID',
        help="This is global and unique ID given to a user across whole ebay. "
        "Warning: Editing this might result in duplication of parties on next"
        " import"
    )

    @classmethod
    def validate(cls, parties):
        """
        Validate sale channel
        """
        super(Party, cls).validate(parties)

        for party in parties:
            party.check_unique_ebay_user_id()

    def check_unique_ebay_user_id(self):
        """
        Check if ebay user id is unique for each party
        """
        if not self.ebay_user_id:
            return
        if self.search([
            ('ebay_user_id', '=', self.ebay_user_id),
            ('id', '!=', self.id)
        ]):
            self.raise_user_error('unique_ebay_user_id')

    @classmethod
    def __setup__(cls):
        """
        Setup the class before adding to pool
        """
        super(Party, cls).__setup__()
        cls._error_messages.update({
            'account_not_found': 'eBay Account does not exist in context',
            'unique_ebay_user_id': 'eBay User ID must be unique for party',
        })

    @classmethod
    def find_or_create_using_ebay_id(cls, ebay_user_id, item_id=None):
        """
        This method tries to find the party with the ebay ID first and
        if not found it will fetch the info from ebay and create a new
        party with the data from ebay using create_using_ebay_data

        :param ebay_user_id: User ID sent by ebay
        :param item_id: This is needed when there is a relationship between
            the buyer and seller. eBay has security which allows a seller
            to fetch the information of a buyer via API only when there is
            a seller-buyer relationship between both via some item.
            If this item is not passed, then ebay would not return important
            informations like eMail etc.
        :return: Active record of record created/found
        """
        SaleChannel = Pool().get('sale.channel')

        ebay_parties = cls.search([
            ('ebay_user_id', '=', ebay_user_id),
        ])

        if ebay_parties:
            return ebay_parties[0]

        ebay_channel = SaleChannel(Transaction().context['current_channel'])
        ebay_channel.validate_ebay_channel()

        api = ebay_channel.get_ebay_trading_api()

        filters = {'UserID': ebay_user_id}
        if item_id:
            filters['ItemID'] = item_id
        user_data = api.execute('GetUser', filters).dict()

        return cls.create_using_ebay_data(user_data)

    @classmethod
    def create_using_ebay_data(cls, ebay_data):
        """
        Creates record of customer values sent by ebay

        :param ebay_data: Dictionary of values for customer sent by ebay
                          Ref: http://developer.ebay.com/DevZone/XML/docs/
                                  Reference/eBay/GetUser.html#Response
        :return: Active record of record created
        """
        return cls.create([{
            # eBay wont expose the name of the buyer to the seller.
            # What we get is the name in the shipping address in the sale order
            # and that we have no way to be sure if that is the address and
            # name of buyer or someone else.
            # Hence, we use UserID for both name and ebay_user_id.
            # This allows the user a flexibility to edit the name later
            'name': ebay_data['User']['UserID'],
            'ebay_user_id': ebay_data['User']['UserID'],
            'contact_mechanisms': [
                ('create', [{
                    # TODO: Handle invalid request
                    'email': ebay_data['User']['Email']
                }])
            ]
        }])[0]

    def add_phone_using_ebay_data(self, ebay_phone):
        """
        Add contact mechanism for party
        """
        ContactMechanism = Pool().get('party.contact_mechanism')

        # Create phone as contact mechanism
        if not ContactMechanism.search([
            ('party', '=', self.id),
            ('type', 'in', ['phone', 'mobile']),
            ('value', '=', ebay_phone),
        ]):
            ContactMechanism.create([{
                'party': self.id,
                'type': 'phone',
                'value': ebay_phone,
            }])

    def get_address_from_ebay_data(self, address_data):
        """
        Return address instance for data fetched from ebay
        """
        Address = Pool().get('party.address')
        Country = Pool().get('country.country')
        Subdivision = Pool().get('country.subdivision')

        country, = Country.search([
            ('code', '=', address_data['Country'])
        ], limit=1)
        subdivision = Subdivision.search_using_ebay_state(
            address_data['StateOrProvince'], country
        )

        return Address(
            party=self.id,
            name=address_data['Name'],
            street=address_data['Street1'],
            streetbis=(
                address_data.get('Street2') or None
            ),
            zip=address_data['PostalCode'],
            city=address_data['CityName'],
            country=country.id,
            subdivision=subdivision.id,
        )

    def find_or_create_address_using_ebay_data(self, address_data):
        """
        Look for the address in tryton corresponding to the address data fecthed
        from ebay.
        If found, return the same else create a new one and return that.

        :param address_data: Dictionary of address data from ebay
        :return: Active record of address created/found
        """
        ebay_address = self.get_address_from_ebay_data(address_data)

        for address in self.addresses:
            if address.is_match_found(ebay_address):
                return address

        # No match found. Create new one.
        ebay_address.save()
        return ebay_address


class Address:
    "Address"
    __name__ = 'party.address'

    def is_match_found(self, ebay_address):
        """
        Match the current address with the ebay address.
        Match all the fields of the address, i.e., streets, city, subdivision
        and country. For any deviation in any field, returns False.

        :param address_data: Dictionary of address data from ebay
                             Ref: http://developer.ebay.com/DevZone/XML/docs/
                                     Reference/eBay/GetUser.html#Response
        :return: True if address matches else False
        """
        return all([
            self.name == ebay_address.name,
            self.street == ebay_address.street,
            self.streetbis == ebay_address.streetbis,
            self.zip == ebay_address.zip,
            self.city == ebay_address.city,
            self.country == ebay_address.country,
            self.subdivision == ebay_address.subdivision,
        ])



"""
    test_party

    Tests party

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import os
import sys
import unittest
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__,
        '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import trytond.tests.test_tryton
from trytond.tests.test_tryton import POOL, USER, DB_NAME, CONTEXT
from test_base import TestBase, load_json
from trytond.transaction import Transaction


class TestProduct(TestBase):
    """
    Tests product
    """

    def test0010_import_product(self):
        """
        Test the import of simple product using eBay Data
        """
        Product = POOL.get('product.product')
        Listing = POOL.get('product.product.channel_listing')

        with Transaction().start(DB_NAME, USER, CONTEXT) as txn:
            self.setup_defaults()

            with txn.set_context({
                'current_channel': self.ebay_channel.id,
                'company': self.company,
            }):

                products_before_import = Product.search([], count=True)

                ebay_data = load_json('products', '110162956809')

                # We need to have SKU in ebay_data
                ebay_data['Item'].update({'SKU': 'test_sku1'})

                product = Product.create_using_ebay_data(
                    ebay_data
                )
                self.assertEqual(
                    product.template.name, 'Iphone'
                )

                products_after_import = Product.search([], count=True)
                self.assertTrue(products_after_import > products_before_import)

                # Create Listing for above product
                # Listing is required because if product is not found
                # then a new product is created from API request
                Listing(
                    product=product,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data['Item']['ItemID'],
                ).save()

                self.assertEqual(
                    product, self.ebay_channel.import_product(
                        ebay_data['Item']['SKU'],
                        product_data={'ItemID': ebay_data['Item']['ItemID']}
                    )
                )

    def test0020_create_product_duplicate(self):
        """
        Tests if items imported from ebay is duplicated as product in tryton
        """
        Product = POOL.get('product.product')
        Listing = POOL.get('product.product.channel_listing')

        with Transaction().start(DB_NAME, USER, CONTEXT) as txn:

            self.setup_defaults()

            with txn.set_context({
                'current_channel': self.ebay_channel.id,
                'company': self.company,
            }):
                self.assertEqual(Product.search([], count=True), 0)

                ebay_data = load_json('products', '110162956809')

                # We need to have SKU in ebay_data
                ebay_data['Item'].update({'SKU': 'test_sku2'})

                # Create product directly (assume its a new product)
                product = Product.create_using_ebay_data(
                    ebay_data
                )

                self.assert_(product)
                self.assertEqual(Product.search([], count=True), 1)

                # Listing is required because if product is not found
                # then a new product is created from API request
                Listing(
                    product=product,
                    channel=self.ebay_channel,
                    product_identifier=ebay_data['Item']['ItemID']
                ).save()

                # Create again
                product = self.ebay_channel.import_product(
                    ebay_data['Item']['SKU'],
                    product_data={'ItemID': ebay_data['Item']['ItemID']}
                )

                self.assertEqual(Product.search([], count=True), 1)


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestProduct)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    test_party

    Tests party

    :copyright: (c) 2013-2015 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import os
import sys
import unittest
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__,
        '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import trytond.tests.test_tryton
from trytond.tests.test_tryton import USER, DB_NAME, CONTEXT
from test_base import TestBase, load_json
from trytond.transaction import Transaction
from trytond.exceptions import UserError


class TestParty(TestBase):
    """
    Tests party
    """

    def test0010_create_party(self):
        """
        Tests if users imported from ebay is created as party in tryton
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):

            self.setup_defaults()

            ebay_data = load_json('users', 'testuser_ritu123')

            self.assertFalse(
                self.Party.search([
                    ('name', '=', 'testuser_ritu123')
                ])
            )

            # Create party
            party = self.Party.create_using_ebay_data(ebay_data)
            self.assert_(party)

            self.assertTrue(
                self.Party.search([
                    ('name', '=', 'testuser_ritu123')
                ])
            )
            party, = self.Party.search([
                ('name', '=', 'testuser_ritu123')
            ])
            self.assertTrue(len(party.contact_mechanisms), 1)
            self.assertTrue(party.contact_mechanisms[0].email)

            # Create party with same data again and it will raise error
            with self.assertRaises(UserError):
                self.Party.create_using_ebay_data(ebay_data)

    def test0030_import_addresses_from_ebay(self):
        """
        Test address import as party addresses and make sure no duplication
        is there.
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):

            self.setup_defaults()

            # Load json of address data
            ebay_data = load_json('users', 'testuser_ritu123')

            order_data = load_json(
                'orders', '283054010'
            )['OrderArray']['Order'][0]
            address_data = order_data['ShippingAddress']

            party = self.Party.create_using_ebay_data(ebay_data)

            # Check party address before address import
            self.assertFalse(party.addresses)

            # Import address for party1 from ebay
            address1 = party.find_or_create_address_using_ebay_data(
                address_data
            )

            # Check address after import
            self.assertEqual(len(party.addresses), 1)
            self.assertEqual(address1.party, party)
            self.assertEqual(
                address1.name, address_data['Name']
            )
            self.assertEqual(address1.street, address_data['Street1'])
            self.assertEqual(address1.zip, address_data['PostalCode'])
            self.assertEqual(address1.city, address_data['CityName'])
            self.assertEqual(
                address1.country.code, address_data['Country']
            )
            self.assertEqual(
                address1.subdivision.name.lower(), 'washington'
            )

            # Try to import same address again. and it wont create new one
            address2 = party.find_or_create_address_using_ebay_data(
                address_data
            )
            self.assertEqual(address1, address2)
            self.assertEqual(len(party.addresses), 1)

    def test0035_import_phone_from_ebay(self):
        """
        Test address import as party addresses and make sure no duplication
        is there.
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):

            self.setup_defaults()

            # Load json of address data
            order_data = load_json(
                'orders', '283054010'
            )['OrderArray']['Order'][0]
            address_data = order_data['ShippingAddress']

            # Check party phone before import
            self.assertFalse(self.party.phone)
            self.assertEqual(len(self.party.contact_mechanisms), 0)

            # Add phone to party using ebay data
            self.party.add_phone_using_ebay_data(
                address_data['Phone']
            )
            self.assertTrue(self.party.phone)

            self.assertEqual(len(self.party.contact_mechanisms), 1)

            # Add same phone again to party using ebay data, it wont
            # create new one
            self.party.add_phone_using_ebay_data(
                address_data['Phone']
            )
            self.assertEqual(len(self.party.contact_mechanisms), 1)

    def test0040_match_address(self):
        """
        Tests if address matching works as expected
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):

            self.setup_defaults()

            self.Subdivision.create([{
                'name': 'New Delhi',
                'code': 'IN-DL',
                'type': 'state',
                'country': self.country_in.id,
            }])
            self.Subdivision.create([{
                'name': 'Goa',
                'code': 'IN-GA',
                'type': 'state',
                'country': self.country_in.id,
            }])
            self.Subdivision.create([{
                'name': 'New York',
                'code': 'US-NY',
                'type': 'state',
                'country': self.country_us.id,
            }])

            # Import address for self.party from ebay
            address = self.party.find_or_create_address_using_ebay_data(
                load_json('addresses', '1a')
            )

            # Same address imported again
            self.assertTrue(
                address.is_match_found(
                    self.party.get_address_from_ebay_data(
                        load_json('addresses', '1b')
                    )
                )
            )

            # Similar with different country and state
            self.assertFalse(
                address.is_match_found(
                    self.party.get_address_from_ebay_data(
                        load_json('addresses', '1c')
                    )
                )
            )

            # Similar with different state
            self.assertFalse(
                address.is_match_found(
                    self.party.get_address_from_ebay_data(
                        load_json('addresses', '1d')
                    )
                )
            )

            # Similar with different city
            self.assertFalse(
                address.is_match_found(
                    self.party.get_address_from_ebay_data(
                        load_json('addresses', '1e')
                    )
                )
            )

            # Similar with different street
            self.assertFalse(
                address.is_match_found(
                    self.party.get_address_from_ebay_data(
                        load_json('addresses', '1f')
                    )
                )
            )

    def test0050_check_unique_ebay_user_id(self):
        """
        Tests if error is raised for duplicate ebay user id for party
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):

            self.setup_defaults()

            # Create party with ebay user ID as None
            party1, = self.Party.create([{
                'name': 'Test Party 1',
                'ebay_user_id': None
            }])

            self.assert_(party1)

            # Create party with ebay user ID as None again
            # Should not raise error
            party2, = self.Party.create([{
                'name': 'Test Party 2',
                'ebay_user_id': None
            }])
            self.assert_(party2)

            # Create party with ebay user ID
            party3, = self.Party.create([{
                'name': 'Test Party 3',
                'ebay_user_id': 'EBAYTEST',
            }])
            self.assert_(party3)

            # Create party with same ebay user ID again
            # Should raise error
            with self.assertRaises(UserError):
                party4, = self.Party.create([{
                    'name': 'Test Party 4',
                    'ebay_user_id': 'EBAYTEST',
                }])


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestParty)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    test_country

    Tests country

    :copyright: (c) 2013 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
import sys
import os
DIR = os.path.abspath(os.path.normpath(
    os.path.join(
        __file__, '..', '..', '..', '..', '..', 'trytond'
    )
))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))

import unittest

import trytond.tests.test_tryton
from trytond.tests.test_tryton import DB_NAME, USER, CONTEXT
from trytond.transaction import Transaction
from trytond.exceptions import UserError
from tests.test_base import TestBase


class TestCountry(TestBase):
    """
    Tests country
    """

    def test_0010_search_state_with_valid_code(self):
        """
        Tests if state can be searched using ebay region
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            country, = self.Country.search([
                ('code', '=', 'US')
            ])
            subdivision, = self.Subdivision.search([
                ('code', '=', 'US-FL')
            ])

            self.assertEqual(
                self.Subdivision.search_using_ebay_state('FL', country),
                subdivision
            )

    def test_0015_search_state_with_valid_name(self):
        """
        Tests if state can be searched using ebay region
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            country, = self.Country.search([
                ('code', '=', 'US')
            ])
            subdivision, = self.Subdivision.search([
                ('code', '=', 'US-FL')
            ])

            self.assertEqual(
                self.Subdivision.search_using_ebay_state('FlorIda', country),
                subdivision
            )

    def test_0020_search_state_with_invalid_region(self):
        """
        Tests if error is raised for searching state with invalid region
        """
        with Transaction().start(DB_NAME, USER, CONTEXT):
            self.setup_defaults()

            country, = self.Country.search([
                ('code', '=', 'US')
            ])

            code = 'abc'

            self.assertFalse(
                self.Subdivision.search([
                    ('code', '=', code),
                    ('country', '=', country.id)
                ])
            )

            self.assertRaises(
                UserError,
                self.Subdivision.search_using_ebay_state, code, country
            )


def suite():
    """
    Test Suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestCountry)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())
